#ifndef __NLL_FORMULA_H_INCLUDED__
#define __NLL_FORMULA_H_INCLUDED__

typedef enum {
  ELEMENT_TYPE_NONE,
  ELEMENT_TYPE_INTEGER,
  ELEMENT_TYPE_STRING,
#ifdef NLL_FLOATING_POINT
  ELEMENT_TYPE_FLOAT,
#endif
  ELEMENT_TYPE_VARIABLE,
  ELEMENT_TYPE_OPERATOR,
  ELEMENT_TYPE_ELEMENTS,
  ELEMENT_TYPE_LABEL,
  ELEMENT_TYPE_NAME,
} element_type_t;

typedef enum {
  OPERATOR_TYPE_NONE = 0 ,
  OPERATOR_TYPE_NOP      ,
  OPERATOR_TYPE_LSHIFTEQ ,
  OPERATOR_TYPE_RSHIFTEQ ,
  OPERATOR_TYPE_INC      ,
  OPERATOR_TYPE_DEC      ,
  OPERATOR_TYPE_INCA     ,
  OPERATOR_TYPE_DECA     ,
  OPERATOR_TYPE_LSHIFT   ,
  OPERATOR_TYPE_RSHIFT   ,
  OPERATOR_TYPE_LE       ,
  OPERATOR_TYPE_GE       ,
  OPERATOR_TYPE_EQ       ,
  OPERATOR_TYPE_NE       ,
  OPERATOR_TYPE_LOGAND   ,
  OPERATOR_TYPE_LOGOR    ,
  OPERATOR_TYPE_LOGCAND  ,
  OPERATOR_TYPE_LOGCOR   ,
  OPERATOR_TYPE_ADDEQ    ,
  OPERATOR_TYPE_SUBEQ    ,
  OPERATOR_TYPE_MULEQ    ,
  OPERATOR_TYPE_DIVEQ    ,
  OPERATOR_TYPE_MODEQ    ,
  OPERATOR_TYPE_ANDEQ    ,
  OPERATOR_TYPE_XOREQ    ,
  OPERATOR_TYPE_OREQ     ,
  OPERATOR_TYPE_NOT      ,
  OPERATOR_TYPE_INV      ,
  OPERATOR_TYPE_PLUS     ,
  OPERATOR_TYPE_MINUS    ,
  OPERATOR_TYPE_PUSH     ,
  OPERATOR_TYPE_POP      ,
  OPERATOR_TYPE_POINTER  ,
  OPERATOR_TYPE_ADDRESS  ,
  OPERATOR_TYPE_NUMBER   ,
  OPERATOR_TYPE_VALUES   ,
  OPERATOR_TYPE_BOOLEAN  ,
  OPERATOR_TYPE_MUL      ,
  OPERATOR_TYPE_DIV      ,
  OPERATOR_TYPE_MOD      ,
  OPERATOR_TYPE_ADD      ,
  OPERATOR_TYPE_SUB      ,
  OPERATOR_TYPE_LT       ,
  OPERATOR_TYPE_GT       ,
  OPERATOR_TYPE_XOR      ,
  OPERATOR_TYPE_AND      ,
  OPERATOR_TYPE_OR       ,
  OPERATOR_TYPE_COND     ,
  OPERATOR_TYPE_AT       ,
  OPERATOR_TYPE_SUBST    ,
  OPERATOR_TYPE_REFER    ,
  OPERATOR_TYPE_STATIC   ,
  OPERATOR_TYPE_ARROW    ,
  OPERATOR_TYPE_DOT      ,
  OPERATOR_TYPE_ARRAY    ,
  OPERATOR_TYPE_VECTOR   ,
  OPERATOR_TYPE_COMMA    ,
  OPERATOR_TYPE_CONT     ,
  OPERATOR_TYPE_FUNCTION ,
  OPERATOR_TYPE_LABELCALL,
} operator_type_t;

typedef enum {
  OPERATOR_SUBTYPE_NONE = 0,
  OPERATOR_SUBTYPE_MONADIC,
  OPERATOR_SUBTYPE_MON_BIN,
  OPERATOR_SUBTYPE_BINARY,
} operator_subtype_t;

typedef struct operator {
  char *word;
  int priority;
  operator_type_t type;
  operator_subtype_t subtype;
  unsigned int flags;
#define OPERATOR_FLAG_RL     (1 <<  0)
#define OPERATOR_FLAG_LR     (1 <<  1)
#define OPERATOR_FLAG_EFFECT (1 <<  2)
} *operator_t;

typedef union element_param {
  struct {
    variable_t variable;
  } variable;
  struct {
    struct operator *op;
    function_t function;
    label_t label;
    element_t args[2];
  } operator;
  struct {
    element_t head;
  } elements;
  struct {
    char name[SYMBOL_NAMELEN + 1];
  } name;
} *element_param_t;

struct element {
  struct element *next;
  element_type_t type;
  value_t value;
  union element_param param;
};

int formula_check(void);
int formula_init(void);
int formula_done(void);
int formula_free(element_t *elementp);
int formula_clean(element_t element);
int formula_parse(char *formula, char **endp, element_t *elementp, const char *terminator);
int formula_proc(element_t element, value_t *valuep);
int formula_dump(FILE *fp, element_t element);

#endif
